// Copyright (C) 2007 Dave Griffiths
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <escheme.h>
#include <map>
#include "VideoTexture.h"

#undef MZ_GC_DECL_REG
#undef MZ_GC_UNREG
#define MZ_GC_DECL_REG(size) void *__gc_var_stack__[size+2] = { (void*)0, (void*)size };
#define MZ_GC_UNREG() (GC_variable_stack = (void**)__gc_var_stack__[0])

using namespace std;

map<int,VideoTexture *> VideoTextures;

Scheme_Object *clear(int argc, Scheme_Object **argv)
{
	for (map<int,VideoTexture *>::iterator i=VideoTextures.begin(); 
		i!=VideoTextures.end(); i++)
	{
		delete i->second;
	}
	VideoTextures.clear();
	
	return scheme_void;
}

Scheme_Object *load(int argc, Scheme_Object **argv)
{
	char *filename=NULL;
	MZ_GC_DECL_REG(2); 
	MZ_GC_VAR_IN_REG(0, argv); 
	MZ_GC_VAR_IN_REG(1, filename); 
	MZ_GC_REG();	

	if (!SCHEME_CHAR_STRINGP(argv[0])) scheme_wrong_type("video-texture-make", "string", 0, argc, argv);
	
	filename=scheme_utf8_encode_to_buffer(SCHEME_CHAR_STR_VAL(argv[0]),SCHEME_CHAR_STRLEN_VAL(argv[0]),NULL,0);
	
	VideoTexture *vt = new VideoTexture(filename);
	VideoTextures[vt->GetID()]=vt;
	
	MZ_GC_UNREG(); 
	return scheme_make_integer_value(vt->GetID());
}

Scheme_Object *update(int argc, Scheme_Object **argv)
{
	MZ_GC_DECL_REG(1); 
	MZ_GC_VAR_IN_REG(0, argv); 
	MZ_GC_REG();	
	if (!SCHEME_NUMBERP(argv[0])) scheme_wrong_type("video-texture-update", "number", 0, argc, argv);
	
	unsigned int id=(unsigned int)scheme_real_to_double(argv[0]);

	map<int,VideoTexture *>::iterator i=VideoTextures.find(id);
	if (i!=VideoTextures.end())
	{
		i->second->Update();
	}
	
	MZ_GC_UNREG(); 
	return scheme_void;
}

/////////////////////

Scheme_Object *scheme_reload(Scheme_Env *env)
{
	// add all the modules from this extension
	Scheme_Env *menv=scheme_primitive_module(scheme_intern_symbol("fluxus-video-texture"), env);

	scheme_add_global("clear-video-textures", scheme_make_prim_w_arity(clear, "clear-video-textures", 0, 0), menv);
	scheme_add_global("load-video-texture", scheme_make_prim_w_arity(load, "load-video-texture", 1, 1), menv);
	scheme_add_global("update-video-texture", scheme_make_prim_w_arity(update, "update-video-texture", 1, 1), menv);

	scheme_finish_primitive_module(menv);	
	
	return scheme_void;
}

Scheme_Object *scheme_initialize(Scheme_Env *env)
{
	return scheme_reload(env);
}

Scheme_Object *scheme_module_name()
{
	return scheme_intern_symbol("fluxus-video-texture");
}
